/*
 *		fimg-core.c
 *
 *
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  <unistd.h>
#include  <string.h>
#include  <math.h>

#include  "../floatimg.h"

extern int verbosity;		/* must be declared around main() */

/* ---------------------------------------------------------------- */
float fimg_get_maxvalue(FloatImg *head)
{
float		maxval;
int		foo;

if (head->type != FIMG_TYPE_RGB && head->type != FIMG_TYPE_GRAY) {
	fprintf(stderr, "%s : type %d invalide\n",
				__func__, head->type);
	return nanf("wtf ?");
	}

maxval = 0.0;		/* no negative values allowed */

switch (head->type) {
	case FIMG_TYPE_RGB:
		for (foo=0; foo<(head->width*head->height); foo++) {
		if (head->R[foo] > maxval) maxval = head->R[foo];
		if (head->G[foo] > maxval) maxval = head->G[foo];
		if (head->B[foo] > maxval) maxval = head->B[foo];
		}
	case FIMG_TYPE_GRAY:
		for (foo=0; foo<(head->width*head->height); foo++) {
		if (head->R[foo] > maxval) maxval = head->R[foo];
		}
	}

return maxval;
}
/* ---------------------------------------------------------------- */
int fimg_meanvalues(FloatImg *head, float means[4])
{
int		idx, surface;
double		accus[4];

surface = head->width * head->height;
if (surface < 1)	return -1;

memset(accus, 0, 4*sizeof(double));

for (idx=0; idx<surface; idx++) {
	accus[0] += head->R[idx];
	if (head->type > 2) {
		accus[1] += head->G[idx];
		accus[2] += head->B[idx];
		}
	}

for (idx=0; idx<4; idx++) {
	means[idx] = (float)(accus[idx]/(double)surface);
	}

return 0;
}
/* ---------------------------------------------------------------- */
/* d'après Wikipedia Fr :
                        | c = 0
                        | s = x1
                        | pour j de 2 à n
                        |   s = s+xj
                        |   c = c+(j xj − s)2/(j(j−1))
                        | renvoyer c/n

Mais c,a ne semble pas etre la bonne methode. Il faut aller voir :
https://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
*/



/* ---------------------------------------------------------------- */
/*
 *	more elaborate functions are in fimg-2gray.c
 */
int fimg_to_gray(FloatImg *head)
{
float		add;
int		foo;

if (head->type != FIMG_TYPE_RGB) {
	fprintf(stderr, "%s : type %d invalide\n",
				__func__, head->type);
	return -3;
	}

for (foo=0; foo<(head->width*head->height); foo++) {
	add  = head->R[foo];
	add += head->G[foo];
	add += head->B[foo];
	head->R[foo] = head->G[foo] = head->B[foo] = add / 3.0;
	}
return -1;
}
/* ---------------------------------------------------------------- */
void fimg_add_cste(FloatImg *fi, float value)
{
int		nbre, idx;

if (fi->type != FIMG_TYPE_RGB) {
	fprintf(stderr, "%s : type %d invalide\n",
				__func__, fi->type);
	return;
	}

nbre = fi->width * fi->height * fi->type;
#if DEBUG_LEVEL
fprintf(stderr, "%s, nbre is %d\n", __func__, nbre);
#endif
for (idx=0; idx<nbre; idx++) {
	fi->R[idx] += value;
	}
}
/* ---------------------------------------------------------------- */
int fimg_count_negativ(FloatImg *fi)
{
int		nbre, idx;
int		count;

if (fi->type != FIMG_TYPE_RGB) {
	fprintf(stderr, "%s : type %d invalide\n",
				__func__, fi->type);
	return -1;
	}

nbre = fi->width * fi->height * fi->type;

count = 0;
for (idx=0; idx<nbre; idx++) {
	if (fi->R[idx] < 0.0) count++;
	}

return count;
}
/* ---------------------------------------------------------------- */
void  fimg_mul_cste(FloatImg *fi, float value)
{
int		nbre, idx;

if ( (fi->type != FIMG_TYPE_RGB) && (fi->type != FIMG_TYPE_GRAY) ) {
	fprintf(stderr, "%s : type %d invalide\n",
				__func__, fi->type);
	return;
	}

nbre = fi->width * fi->height * fi->type;
#if DEBUG_LEVEL
fprintf(stderr, "%s, nbre of datum is %d\n", __func__, nbre);
#endif
for (idx=0; idx<nbre; idx++) {
	fi->R[idx] *= value;
	}
}
/* ---------------------------------------------------------------- */
int fimg_normalize(FloatImg *fi, double maxima, int notused)
{
double		coef;

if (fi->type != FIMG_TYPE_RGB) {
	fprintf(stderr, "%s : type %d invalide\n",
				__func__, fi->type);
	return -99;
	}

if (fi->count < 1) {
	fprintf(stderr, "%s : count %d is invalid\n", __func__, fi->count);
	return -98;
	}

/*
 *		mmmm, is this real ?
 *	how to accuratly check the value of 'I.fval' ?
 */
coef = 1.0 / ((double)fi->count * (double)fi->fval);

if (verbosity) {
	fprintf(stderr, "image @    %p\n", fi);
	fprintf(stderr, "fval       %f\n", fi->fval);
	fprintf(stderr, "count      %d\n", fi->count);
	fprintf(stderr, "coef       %f\n", coef);
	}

fimg_mul_cste(fi, coef);

return 0;
}
/* ---------------------------------------------------------------- */
/*            Warning: this function is _very_ slow                 */
void fimg_drand48(FloatImg *fi, float kmul)
{
int		nbre, idx;

#if DEBUG_LEVEL
fprintf(stderr, ">>> %s ( %p %g )\n", __func__, fi, kmul);
#endif

if (fi->type != FIMG_TYPE_RGB) {
	fprintf(stderr, "%s : type %d invalide\n",
				__func__, fi->type);
	return;
	}
nbre = fi->width * fi->height;
for (idx=0; idx<nbre; idx++) {
	fi->R[idx] = drand48() * kmul;
	fi->G[idx] = drand48() * kmul;
	fi->B[idx] = drand48() * kmul;
	}

}
/* ---------------------------------------------------------------- */


